@@ -92,31 +92,55 @@ def pipe_reply(reply_end, length_on_wavelength):
     angle = FOUR_PI*length_on_wavelength
     return complex(math.cos(angle),math.sin(angle)) * reply_end
 
-
-def junction_reply(area, areas, replies):
-    total_area = area + sum(areas)
-    
-    return area / (total_area*(0.5 - sum( a/(total_area*(1.0/r+1.0)) for a,r in zip(areas, replies) ))) - 1.0
-    
-    #Equivalently?
-    #x = area / (total_area*(0.5 - sum( a/(total_area*(r+1.0)) for a,r in zip(areas, replies) ))) - 1.0
-    #return 1/x
+#
+#def junction_reply(area, areas, replies):
+#    total_area = area + sum(areas)
+#    
+#    return area / (total_area*(0.5 - sum( a/(total_area*(1.0/r+1.0)) for a,r in zip(areas, replies) ))) - 1.0
+#    
+#    #Equivalently?
+#    #x = area / (total_area*(0.5 - sum( a/(total_area*(r+1.0)) for a,r in zip(areas, replies) ))) - 1.0
+#    #return 1/x
+#
+#
+#def junction2_reply(area, area1, reply1):
+#    total_area = area + area1
+#    
+#    return area / (total_area*(0.5 - area1/(total_area*(1.0/reply1+1.0)))) - 1.0
+#
+#
+#def junction3_reply(area, area1, area2, reply1, reply2):
+#    total_area = area + area1 + area2
+#    
+#    return area / (total_area*(
+#        0.5 
+#        - area1/(total_area*(1.0/reply1+1.0))
+#        - area2/(total_area*(1.0/reply2+1.0))
+#    )) - 1.0
+#
 
 
-def junction2_reply(area, area1, reply1):
-    total_area = area + area1
-    
-    return area / (total_area*(0.5 - area1/(total_area*(1.0/reply1+1.0)))) - 1.0
+
+def junction2_reply(a0, a1, r1):
+    """ Relative phase reply for pipe 0 of two pipe junction
+        with pipe cross section areas a0 and a1
+        and pipe 1 relative phase reply r1.    
+    """
+    pjunc = 2.0*a0 / (a0-a1*((r1-1.0)/(r1+1.0)))
+    mag1 = abs( pjunc / (r1+1.0) )    
+    return pjunc-1.0, mag1
 
 
-def junction3_reply(area, area1, area2, reply1, reply2):
-    total_area = area + area1 + area2
-    
-    return area / (total_area*(
-        0.5 
-        - area1/(total_area*(1.0/reply1+1.0))
-        - area2/(total_area*(1.0/reply2+1.0))
-    )) - 1.0
+def junction3_reply(a0,a1,a2, r1,r2):
+    """ Relative phase reply for pipe 0 of three pipe junction
+        with pipe cross section areas a0, a1 and a2
+        and relative phase replies r1 and r2.    
+    """
+    pjunc = 2.0*a0 / (a0-a1*((r1-1.0)/(r1+1.0))-a2*((r2-1.0)/(r2+1.0)))    
+    mag1 = abs( pjunc / (r1+1.0) )
+    mag2 = abs( pjunc / (r2+1.0) )
+    return pjunc-1.0, mag1, mag2
+
 
 
 def end_flange_length_correction(outer_diameter, inner_diameter):
@@ -190,10 +214,11 @@ class Instrument:
         
         position = -end_flange_length_correction(self.outer(0.0,True),self.stepped_inner(0.0,True))
         diameter = self.stepped_inner(0.0, True)
+        self.initial_emission = [ circle_area(diameter) ]
         for pos, action, index in events:
             length = pos-position
             
-            def func(reply_end, wavelength, fingers, length=length):
+            def func(reply_end, wavelength, fingers, emission, length=length):
                 return pipe_reply(reply_end, length/wavelength)
             self.actions.append(func)
             
@@ -205,8 +230,12 @@ class Instrument:
                 diameter = self.stepped_inner.high[index]
                 area = circle_area(diameter)
                 
-                def func(reply, wavelength, fingers, area=area, area1=area1):
-                    return junction2_reply(area, area1, reply)
+                def func(reply, wavelength, fingers, emission, area=area, area1=area1):
+                    new_reply, mag1 = junction2_reply(area, area1, reply)
+                    if emission is not None:
+                        for i in xrange(len(emission)):
+                            emission[i] *= mag1
+                    return new_reply
                 self.actions.append(func)
                 
             elif action == 'hole':
@@ -222,7 +251,7 @@ class Instrument:
                 open_length = true_length + hole_length_correction(hole_diameter, diameter, False) 
                 closed_length = true_length + hole_length_correction(hole_diameter, diameter, True)
 
-                def func(reply, wavelength, fingers, 
+                def func(reply, wavelength, fingers, emission,
                          area=area,hole_area=hole_area,
                          open_length=open_length,closed_length=closed_length,
                          index=index):
@@ -230,15 +259,36 @@ class Instrument:
                         hole_reply = pipe_reply(1.0, closed_length/wavelength)
                     else:
                         hole_reply = pipe_reply(-1.0, open_length/wavelength)
-                    return junction3_reply(area, area, hole_area, reply, hole_reply)
+                    new_reply, mag1, mag2 = junction3_reply(area, area, hole_area, reply, hole_reply)
+                
+                    if emission is not None:
+                        for i in xrange(len(emission)):
+                            emission[i] *= mag1
+                        if not fingers[index]:
+                            emission.append(hole_area*mag2)
+                
+                    return new_reply
+                    
                 self.actions.append(func)
     
+        self.emission_divide = circle_area(diameter)
     
-    def resonance_score(self, w, fingers):
+    
+    def resonance_score(self, w, fingers, calc_emission=False):
         """ A score -1 <= score <= 1, zero if wavelength w resonantes """
         reply = -1.0 #Open end
-        for action in self.actions:
-            reply = action(reply, w, fingers)
+        
+        if not calc_emission:
+            for action in self.actions:
+                reply = action(reply, w, fingers, None)
+        else:
+            emission = list(self.initial_emission)
+            for action in self.actions:
+                reply = action(reply, w, fingers, emission)
+        
+            # Scale by top area
+            for i in xrange(len(emission)):
+                emission[i] /= self.emission_divide
         
         if not self.closed_top: reply *= -1.0
         
@@ -248,10 +298,15 @@ class Instrument:
         angle2 = angle1 - math.pi*2.0
         
         if angle1 < -angle2:
-            return angle1/math.pi
+            result = angle1/math.pi
         else:
-            return angle2/math.pi
+            result = angle2/math.pi
         
+        if not calc_emission:
+            return result
+        else:
+            return result, emission
+    
         
     def true_wavelength_near(self, w, fingers, max_grad, step_cents = 1.0, step_increase = 1.05, max_steps = 100):
         step = pow(2.0, step_cents/1200.0)
@@ -321,7 +376,7 @@ def low_high(vec):
     low = [ ]
     high = [ ]
     for item in vec:
-        if isinstance(item, collections.Sequence):
+        if isinstance(item, tuple) or isinstance(item, list):
            assert len(item) == 2
            low.append(item[0])
            high.append(item[1])
@@ -331,7 +386,7 @@ def low_high(vec):
     return low, high
 
 def describe_low_high(item):
-    if isinstance(item, collections.Sequence):
+    if isinstance(item, tuple) or isinstance(item, list):
        return '%.1f->%.1f' % item
     else:
        return '%.1f' % item
@@ -361,6 +416,12 @@ def power_scaler(power, value):
 
 
 @config.Int_flag('transpose', 'Transpose instrument by this many semitones.')
+@config.Float_flag('tweak_emission', 
+    'Experimental. '
+    'Add a term to the optimization '
+    'to try to make instrument louder, '
+    'possibly sacrificing being-in-tuneness.'
+    )
 class Instrument_designer(config.Action_with_output_dir):
     instrument_class = Instrument
 
@@ -370,6 +431,8 @@ class Instrument_designer(config.Action_with_output_dir):
 
     transpose = 0
     
+    tweak_emission = 0.0
+    
     initial_length = None
 
     fingerings = [ ]
@@ -427,6 +490,10 @@ class Instrument_designer(config.Action_with_output_dir):
     def min_inner_fraction_sep(self):
         return [ 0.0 ] * (len(self.inner_diameters)-1)
 
+    @property
+    def max_inner_fraction_sep(self):
+        return [ 1.0 ] * (len(self.inner_diameters)-1)
+
     @property
     def initial_outer_fractions(self):
         return [ (i+1.0)/(len(self.outer_diameters)-1) for i in xrange(len(self.outer_diameters)-2) ]
@@ -435,6 +502,10 @@ class Instrument_designer(config.Action_with_output_dir):
     def min_outer_fraction_sep(self):
         return [ 0.0 ] * (len(self.outer_diameters)-1)
 
+    @property
+    def max_outer_fraction_sep(self):
+        return [ 1.0 ] * (len(self.outer_diameters)-1)
+
     @property
     def initial_hole_fractions(self):
         return [ (i+3.0)/(self.n_holes+2) * 0.5 for i in xrange(self.n_holes) ]
@@ -554,52 +625,48 @@ class Instrument_designer(config.Action_with_output_dir):
         """ Return an amount of constraint dissatisfaction """
         scores = [ ]
         
-        #if inst.length < 0: 
-            #score += -inst.length
         scores.append(inst.length)
         
         inners = [0.0]+inst.inner_kinks+[inst.length]
         for i in range(len(inners)-1):
-            check = inners[i+1] - inners[i] - self.min_inner_fraction_sep[i]*inst.length
-            #if check < 0: score += -check 
+            sep = inners[i+1]-inners[i]
+            check = sep - self.min_inner_fraction_sep[i]*inst.length
+            scores.append(check)
+            check = self.max_inner_fraction_sep[i]*inst.length - sep
             scores.append(check)
         
         outers = [0.0]+inst.outer_kinks+[inst.length]
         for i in range(len(outers)-1):
-            check = outers[i+1] - outers[i] - self.min_outer_fraction_sep[i]*inst.length
-            #if check < 0: score += -check
+            sep = outers[i+1]-outers[i]
+            check = sep - self.min_outer_fraction_sep[i]*inst.length
+            scores.append(check)
+            check = self.max_outer_fraction_sep[i]*inst.length - sep
             scores.append(check)
         
         if self.n_holes:
             check = inst.hole_positions[0] - self.bottom_clearance_fraction*inst.length
-            #if check < 0: score += -check
             scores.append(check)
             
             check = (1.0-self.top_clearance_fraction)*inst.length - inst.hole_positions[-1]
-            #if check < 0: score += -check
             scores.append(check)
             
             for i, value in enumerate(self.min_hole_spacing):
                 if value is None: continue
                 check = (inst.hole_positions[i+1]-inst.hole_positions[i]) - value
-                #if check < 0: score += -check
                 scores.append(check)
             
             for i, value in enumerate(self.max_hole_spacing):
                 if value is None: continue
                 check = value - (inst.hole_positions[i+1]-inst.hole_positions[i])
-                #if check < 0: score += -check
                 scores.append(check)                
             
             for i, value in enumerate(self.min_hole_diameters):
                 check = inst.hole_diameters[i] - value
-                #if check < 0: score += -check
                 scores.append(check)
             
             for i, value in enumerate(self.max_hole_diameters):
                 if value is None: continue
                 check = value - inst.hole_diameters[i]
-                #if check < 0: score += -check
                 scores.append(check)
             
             for i, value in enumerate(self.balance):
@@ -607,13 +674,10 @@ class Instrument_designer(config.Action_with_output_dir):
                 check = value * 0.5*(inst.hole_positions[i+2]-inst.hole_positions[i]) - abs(
                     0.5*inst.hole_positions[i]+0.5*inst.hole_positions[i+2]-inst.hole_positions[i+1]
                 )
-                #if check < 0: score += -check
                 scores.append(check)
 
         negscores = [ -item for item in scores if item < 0.0 ]
         negscore = sum(negscores) if negscores else 0.0
-        #posscores = [ (item+1e-30)**-1 for item in scores if item >= 0.0 ]
-        #posscore = 1.0/sum(posscores) if posscores else 1e30
         return negscore
 
     def patch_instrument(self, inst):
@@ -625,6 +689,10 @@ class Instrument_designer(config.Action_with_output_dir):
     
         score = 0.0
         div = 0.0
+
+        emission_score = 0.0
+        #emission_score2 = 0.0
+        emission_div = 0.0
         
         inst.prepare()
         
@@ -635,10 +703,28 @@ class Instrument_designer(config.Action_with_output_dir):
             diff = abs(math.log(w1)-math.log(w2))*s
             #weight = w1
             weight = 1.0
-            score += weight * diff**3 / (1.0 + (diff/20.0)**2)
+            #score += weight * diff**3 / (1.0 + (diff/20.0)**2)
+            score += weight * diff**3
             div += weight
+                        
+            if self.tweak_emission:
+                emission_weight = 1.0 #w1
+                emission_div += emission_weight
+                _, emission = inst.resonance_score(w2,fingers,True)
+                rms = math.sqrt(sum(item*item for item in emission))
+                x = math.log(rms)
+                emission_score += emission_weight * x
+                #emission_score2 += emission_weight * x * x
+                            
             
-        return (score/div)**(1.0/3)
+        result = (score/div)**(1.0/3) 
+        if self.tweak_emission:
+            x = emission_score / emission_div
+            #x2 = emission_score2 / emission_div
+            #var = x2-x*x
+            result += self.tweak_emission * -x #math.sqrt(var)/x
+        return result
+        
         
         #return ( (score/scale) ** (1.0/2) )*100.0
 
@@ -698,17 +784,17 @@ class Instrument_designer(config.Action_with_output_dir):
         patched_instrument = self.patch_instrument(self.instrument)
         patched_instrument.prepare()
         
-        #any_extra = any( item != 0.0 for item in self.hole_extra_height_by_diameter )
-        #if any_extra:
-        #TODO: implement embextra using patch_instrument
-        mod_instrument = self.unpack( state_vec )
-        #mod_instrument.hole_extra_height_by_diameter = [ 0.0 ] * len(self.hole_extra_height_by_diameter)
-        mod_instrument.hole_lengths = [
-            (mod_instrument.outer(mod_instrument.hole_positions[i])
-             - mod_instrument.inner(mod_instrument.hole_positions[i])) * 0.5
-            for i in range(self.n_holes)
-        ]
-        mod_instrument.prepare()
+      #  #any_extra = any( item != 0.0 for item in self.hole_extra_height_by_diameter )
+      #  #if any_extra:
+      #  #TODO: implement embextra using patch_instrument
+      #  mod_instrument = self.unpack( state_vec )
+      #  #mod_instrument.hole_extra_height_by_diameter = [ 0.0 ] * len(self.hole_extra_height_by_diameter)
+      #  mod_instrument.hole_lengths = [
+      #      (mod_instrument.outer(mod_instrument.hole_positions[i])
+      #       - mod_instrument.inner(mod_instrument.hole_positions[i])) * 0.5
+      #      for i in range(self.n_holes)
+      #  ]
+      #  mod_instrument.prepare()
         
         diagram = svg.SVG()
         
@@ -745,7 +831,10 @@ class Instrument_designer(config.Action_with_output_dir):
         diagram.text(text_x + 90.0, min(text_y,-self.instrument.length), '%.1fmm' % self.instrument.length)
 
         text_x = diagram.max_x
+        graph_x = text_x+200
+        emit_x = graph_x+220
         text_y = 0
+        
         for note, fingers in self.fingerings:
             w1 = wavelength(note, self.transpose)
             w2, grad = patched_instrument.true_wavelength_near(w1, fingers, self.max_grad)
@@ -759,7 +848,6 @@ class Instrument_designer(config.Action_with_output_dir):
             probes = [ low * pow(step,i) for i in range(n_probes) ]
             scores = [ patched_instrument.resonance_score(probe, fingers) for probe in probes ]
 
-            graph_x = text_x+200
             points = [ (graph_x+i*width/n_probes,text_y-score*7.0)
                        for i,score in enumerate(scores) ]
             for i in range(len(probes)-1):
@@ -772,6 +860,11 @@ class Instrument_designer(config.Action_with_output_dir):
                   '%5s %s %-4d cents grad=%.1f' % (describe(w1), '     ' if cents == 0 else ' flat' if cents > 0 else 'sharp', abs(cents), grad)
             )
             
+            _, emission = patched_instrument.resonance_score(w2,fingers,True)
+            diagram.text(emit_x, text_y,
+                 ', '.join('%.2f' % item for item in emission)
+                 )
+            
             #if any_extra:
             #w3, grad3 = mod_instrument.true_wavelength_near(w1, fingers, self.max_grad)
             #cents3 = int(round( log2(w3/w1) * 1200.0 ))
@@ -781,19 +874,32 @@ class Instrument_designer(config.Action_with_output_dir):
             
             
             text_y -= 25
+        
+        diagram.text(graph_x, text_y - 10,
+            'Nearby resonances:',
+            color='#000000'
+            )
+        
+        diagram.text(emit_x, text_y - 20,
+            'Volume of sound emission from\n'
+            'instrument end and open holes:',
+            color='#000000',
+            )
             
             
         
-        diagram.text(0.0, 20.0, 'Outer diameters:')
+        text_y -= 50.0 + 10.0 * max(len(self.inner_diameters),len(self.outer_diameters))
+        
+        diagram.text(graph_x-150.0, text_y, 'Outer diameters:', color='#000000')
         kinks = [0.0]+self.instrument.outer_kinks+[self.instrument.length]
         for i, item in enumerate(self.outer_diameters):
-            diagram.text(0.0, 20.0+(len(self.outer_diameters)-i)*10.0, 
+            diagram.text(graph_x-150.0, text_y+10.0+(len(self.outer_diameters)-i)*10.0, 
                 describe_low_high(item) + 'mm at %.1fmm' % kinks[i]) 
         
-        diagram.text(150.0, 20.0, 'Inner diameters:')
+        diagram.text(graph_x, text_y, 'Inner diameters:', color='#000000')
         kinks = [0.0]+self.instrument.inner_kinks+[self.instrument.length]
         for i, item in enumerate(self.inner_diameters):
-            diagram.text(150.0, 20.0+(len(self.inner_diameters)-i)*10.0, 
+            diagram.text(graph_x, text_y+10.0+(len(self.inner_diameters)-i)*10.0, 
                 describe_low_high(item) + 'mm at %.1fmm' % kinks[i]) 
         
         diagram.save( os.path.join(self.output_dir, 'diagram.svg') )
