@@ -7,5 +7,8 @@ class Design_my_whistle(demakein.Design_folk_whistle):
     
     # ... and any other things you want to change ...
 
+
 if __name__ == '__main__':
-    nesoni.run_tool(Design_my_whistle)
\ No newline at end of file
+    nesoni.run_toolbox(
+        [ Design_my_whistle, demakein.Make_whistle ],
+        show_make_flags=False)
